/*
 * Copyright 2006-2010 The JGUIraffe Team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sf.jguiraffe.di.impl;

import net.sf.jguiraffe.di.Dependency;
import net.sf.jguiraffe.di.DependencyProvider;
import net.sf.jguiraffe.di.InjectionException;

/**
 * <p>
 * A class that represents a method invocation.
 * </p>
 * <p>
 * This class stores all data, which is needed for invoking a method; i.e. the
 * method name, the target class, optional information about the data types of
 * the method parameters, and the current parameter values to be passed to the
 * method.
 * </p>
 * <p>
 * Once initialized, an instance is immutable. So it can easily be shared
 * between multiple components and threads without having to care about
 * synchronization issues. The <code>invoke()</code> method actually executes
 * the corresponding method.
 * </p>
 *
 * @author Oliver Heger
 * @version $Id: MethodInvocation.java 195 2010-08-30 19:54:41Z oheger $
 */
public class MethodInvocation extends Invocation implements Invokable
{
    /** Stores the name of the method to invoke. */
    private final String methodName;

    /** Stores a flag whether this is a static method invocation. */
    private final boolean staticInvocation;

    /**
     * Creates a new instance of <code>MethodInvocation</code> for non-static
     * method invocations. This constructor sets the target class to <b>null</b>,
     * so that it can only be derived from the target instance (which prohibits
     * static method invocations).
     *
     * @param methodName the name of the method to be invoked (must not be
     * <b>null</b>)
     * @param paramTypes an array with the parameter types
     * @param paramValues the current parameter values (defined as
     * <code>Dependency</code> objects); this array must not contain <b>null</b>
     * elements
     * @throws IllegalArgumentException if the length of the parameter types
     * array does not match the length of the parameter values array, or if the
     * values array contains <b>null</b> elements, or if the method name is
     * undefined
     */
    public MethodInvocation(String methodName, ClassDescription[] paramTypes,
            Dependency... paramValues)
    {
        this(null, methodName, paramTypes, paramValues);
    }

    /**
     * Creates a new instance of <code>MethodInvocation</code> and initializes
     * it with information about the method to invoke and the target class.
     *
     * @param targetClass the class, on which the method is to be invoked
     * @param methodName the name of the method to be invoked (must not be
     * <b>null</b>)
     * @param paramTypes an array with the parameter types
     * @param paramValues the current parameter values (defined as
     * <code>Dependency</code> objects); this array must not contain <b>null</b>
     * elements
     * @throws IllegalArgumentException if the length of the parameter types
     * array does not match the length of the parameter values array, or if the
     * values array contains <b>null</b> elements, or if the method name is
     * undefined
     * @see Invocation#Invocation(Class, Class[], Dependency...)
     */
    public MethodInvocation(ClassDescription targetClass, String methodName,
            ClassDescription[] paramTypes, Dependency... paramValues)
    {
        this(targetClass, methodName, false, paramTypes, paramValues);
    }

    /**
     * Creates a new instance of <code>MethodInvocation</code> and fully
     * initializes it. This constructor takes all information required for
     * arbitrary method invocations. Refer to the base class for a detailed
     * explanation of the arguments.
     *
     * @param targetClass the class, on which the method is to be invoked
     * @param methodName the name of the method to be invoked (must not be
     * <b>null</b>)
     * @param isStatic determines whether a static method is to be invoked
     * @param paramTypes an array with the parameter types
     * @param paramValues the current parameter values (defined as
     * <code>Dependency</code> objects); this array must not contain <b>null</b>
     * elements
     * @throws IllegalArgumentException if the length of the parameter types
     * array does not match the length of the parameter values array, or if the
     * values array contains <b>null</b> elements, or if the method name is
     * undefined, or if the static flag is <b>true</b>, but no target class is
     * defined
     * @see Invocation#Invocation(Class, Class[], Dependency...)
     */
    public MethodInvocation(ClassDescription targetClass, String methodName,
            boolean isStatic, ClassDescription[] paramTypes,
            Dependency... paramValues)
    {
        super(targetClass, paramTypes, paramValues);
        if (methodName == null)
        {
            throw new IllegalArgumentException("Method name must not be null!");
        }
        if (targetClass == null && isStatic)
        {
            throw new IllegalArgumentException(
                    "Need a target class for a static invocation!");
        }

        this.methodName = methodName;
        staticInvocation = isStatic;
    }

    /**
     * Returns the name of the method to be invoked.
     *
     * @return the method name
     */
    public String getMethodName()
    {
        return methodName;
    }

    /**
     * Returns the static flag. This flag indicates whether a static method is
     * to be invoked.
     *
     * @return the static invocation flag
     */
    public boolean isStaticInvocation()
    {
        return staticInvocation;
    }

    /**
     * Invokes the corresponding method on the specified target instance. The
     * method's result is returned. The behavior of this method depends on the
     * {@link #isStaticInvocation()} flag. If it is set, a passed
     * in target object is ignored and a static method invocation on the target
     * class is performed. Otherwise, if a non <b>null</b> target object is
     * passed in, the target class is derived from this instance (an eventually
     * set target class is ignored).
     *
     * @param depProvider the dependency provider for resolving the parameters
     * (must not be <b>null</b>)
     * @param target the target instance, on which to invoke the method
     * @return the method's return value
     * @throws InjectionException in case of an error
     * @throws IllegalArgumentException if the dependency provider is <b>null</b>
     */
    public Object invoke(DependencyProvider depProvider, Object target)
    {
        if (target == null && getTargetClass() == null)
        {
            throw new InjectionException(
                    "Target class and instance must not both be null!");
        }

        Object[] values = getResolvedParameters(depProvider);
        if (isStaticInvocation() || target == null)
        {
            return depProvider.getInvocationHelper().invokeStaticMethod(
                    getTargetClass().getTargetClass(depProvider),
                    getMethodName(), getParameterClasses(depProvider), values);
        }
        else
        {
            return depProvider.getInvocationHelper().invokeInstanceMethod(
                    target, getMethodName(), getParameterClasses(depProvider),
                    values);
        }
    }

    /**
     * Creates a string with additional information about this invocation. This
     * implementation will output the method name.
     *
     * @param buf the target buffer
     */
    @Override
    protected void invocationInfoToString(StringBuilder buf)
    {
        super.invocationInfoToString(buf);
        if (getTargetClass() != null)
        {
            buf.append('.');
        }
        buf.append(getMethodName());
    }
}
