/*
 * Copyright 2006-2010 The JGUIraffe Team.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sf.jguiraffe.gui.builder.components.tags;

import net.sf.jguiraffe.gui.builder.components.FormBuilderException;
import net.sf.jguiraffe.locators.ClassPathLocator;
import net.sf.jguiraffe.locators.Locator;
import net.sf.jguiraffe.locators.URLLocator;

import org.apache.commons.jelly.JellyTagException;
import org.apache.commons.jelly.MissingAttributeException;
import org.apache.commons.lang.StringUtils;

/**
 * <p>
 * A tag for adding an icon to a component.
 * </p>
 * <p>
 * This tag supports multiple ways for defining an icon:
 * <ul>
 * <li>By specifying an URL using the <code>url</code> attribute.</li>
 * <li>By specifying a resource name using the <code>resource</code> attribute.
 * In this case the icon is searched on the class path.</li>
 * <li>By specifying an arbitrary <code>{@link Locator}</code> pointing to the
 * icon data using the <code>locator</code> attribute. This is actually the
 * generic form of the ways described above. The locator is looked up by its
 * name in the current bean context.</li>
 * <li>With the <code>ref</code> attribute an icon can be referenced that has
 * already been created and stored in the Jelly context (e.g by another
 * <code>icon</code> tag that had a <code>var</code> attribute (see below).</li>
 * </ul>
 * Exactly one of these attributes must be present.
 * </p>
 * <p>
 * The icon is created using the component manager. Then the tag searches for an
 * enclosing tag that implements the <code>{@link IconSupport}</code> interface.
 * If one is found, this tag is passed the icon. With the <code>var</code>
 * attribute the name of a variable can be specified, which is assigned the
 * icon. So it can be reused. At least one of the ways for storing an icon (the
 * <code>var</code> attribute or an <code>IconSupport</code> tag) must be
 * defined, otherwise an exception is thrown.
 * </p>
 *
 * @author Oliver Heger
 * @version $Id: IconTag.java 195 2010-08-30 19:54:41Z oheger $
 */
public class IconTag extends FormBaseTag
{
    /** Stores the URL of the icon. */
    private String url;

    /** Stores the resource name of the icon. */
    private String resource;

    /** Stores the name of a variable, from which the icon should be fetched. */
    private String ref;

    /** Stores the name of a variable, to which the icon should be assigned. */
    private String var;

    /** Stores the name of the locator pointing to the icon data. */
    private String locator;

    /**
     * Returns the resource name of the icon.
     *
     * @return the resource name
     */
    public String getResource()
    {
        return resource;
    }

    /**
     * Setter method for the resource attribute. Specifies the name under which
     * the icon can be found on the classpath. Either this resource name or an
     * URL must be specified.
     *
     * @param resource the resource name
     */
    public void setResource(String resource)
    {
        this.resource = resource;
    }

    /**
     * Returns the URL that points to the icon.
     *
     * @return the URL
     */
    public String getUrl()
    {
        return url;
    }

    /**
     * Setter method for the URL attribute. Specifies an URL from which the icon
     * can be loaded. Either this URL or a resource name must be specified.
     *
     * @param url the URL
     */
    public void setUrl(String url)
    {
        this.url = url;
    }

    /**
     * Returns the name of the variable that stores the icon.
     *
     * @return the name of a variable reference
     */
    public String getRef()
    {
        return ref;
    }

    /**
     * Setter method for the var attribute. If no URL or resource name are
     * specified, the icon can be obtained from a variable.
     *
     * @param ref the attribute's value
     */
    public void setRef(String ref)
    {
        this.ref = ref;
    }

    /**
     * Returns the name of the variable, to which the icon should be assigned.
     *
     * @return the name of a target variable
     */
    public String getVar()
    {
        return var;
    }

    /**
     * Setter method of the var attribute.
     *
     * @param var the attribute's value
     */
    public void setVar(String var)
    {
        this.var = var;
    }

    /**
     * Returns the name of the <code>Locator</code>, from which the icon should
     * be obtained.
     *
     * @return the name of the <code>Locator</code>
     */
    public String getLocator()
    {
        return locator;
    }

    /**
     * Setter method of the <code>locator</code> attribute. With this attribute
     * an arbitrary <code>Locator</code> can be specified, from which the icon
     * data is loaded. The locator is looked up in the current bean context
     * under the name specified here.
     *
     * @param locator the name of <code>Locator</code> with the icon data
     */
    public void setLocator(String locator)
    {
        this.locator = locator;
    }

    /**
     * Executes this tag. Tries to create the icon and set it on an enclosing
     * {@link IconSupport} tag.
     *
     * @throws JellyTagException if an error occurs
     * @throws FormBuilderException if an error occurs
     */
    @Override
    protected void process() throws JellyTagException, FormBuilderException
    {
        int defCount = countIconDefinitions();
        if (defCount < 1)
        {
            throw new MissingAttributeException(
                    "Either url or resource or ref attribute must be set!");
        }
        if (defCount > 1)
        {
            throw new FormBuilderException(
                    "Exactly one of these attributes must be used: "
                            + "url, resource, ref");
        }

        IconSupport parent = (IconSupport) findAncestorWithClass(IconSupport.class);
        if (parent == null && StringUtils.isEmpty(getVar()))
        {
            throw new JellyTagException(
                    "Icon tag must be nested inside an IconSupport tag!");
        }

        Object icon = createIcon();
        if (parent != null)
        {
            parent.setIcon(icon);
        }
        if (getVar() != null)
        {
            getContext().setVariable(getVar(), icon);
        }
    }

    /**
     * Creates the icon. This method is called after all syntactic checks were
     * successful. Here the icon must be created, depending on the defined
     * attributes.
     *
     * @return the icon
     * @throws FormBuilderException if an error occurs
     */
    protected Object createIcon() throws FormBuilderException
    {
        if (getRef() != null)
        {
            Object icon = getContext().getVariable(getRef());
            if (icon == null)
            {
                throw new FormBuilderException(
                        "Cannot find referenced variable: " + getRef());
            }
            return icon;
        }
        else
        {
            return getBuilderData().getComponentManager().createIcon(
                    fetchLocator());
        }
    }

    /**
     * Fetches the locator for the icon. This method is called by
     * <code>createIcon()</code> when no reference attribute is specified, i.e.
     * the icon has to be obtained from a locator. If the <code>locator</code>
     * attribute is specified, the locator is looked up from the current bean
     * context. Otherwise a locator is created from the other convenience
     * attributes (<code>url</code> or <code>resource</code>).
     *
     * @return the <code>Locator</code> that defines the icon
     */
    protected Locator fetchLocator()
    {
        if (getLocator() != null)
        {
            return (Locator) getBuilderData().getBeanContext().getBean(
                    getLocator());
        }
        else if (getResource() != null)
        {
            return ClassPathLocator.getInstance(getResource());
        }
        else
        {
            assert getUrl() != null : "No locator defined!";
            return URLLocator.getInstance(getUrl());
        }
    }

    /**
     * Counts in how many ways the icon is defined. There should be exactly one
     * way.
     *
     * @return the number of attributes that define the icon
     */
    private int countIconDefinitions()
    {
        int count = 0;
        if (!StringUtils.isEmpty(getUrl()))
        {
            count++;
        }
        if (!StringUtils.isEmpty(getResource()))
        {
            count++;
        }
        if (!StringUtils.isEmpty(getRef()))
        {
            count++;
        }
        if (!StringUtils.isEmpty(getLocator()))
        {
            count++;
        }
        return count;
    }
}
